@@ -1,6 +1,6 @@
 from typing import Optional, List, Union
 import time
-from hein_robots.robotics import Location, Twist, Wrench, Cartesian, Orientation, Units
+from hein_robots.robotics import Location, Twist, Wrench, Cartesian, Orientation, Units, Frame
 from hein_robots.base import robot_arms
 from hein_robots.base.actuators import Actuator
 from hein_robots.kinova.kortex import KortexConnection, Base_pb2, ControlConfig_pb2
@@ -10,7 +10,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
     GRIPPER_VELOCITY_STOP_THRESHOLD = 0.01
 
     def __init__(self, host: str = '192.168.1.10', port: int = 10000, username: str = 'admin', password: str = 'admin', connect: bool = True,
-                 default_velocity: float = 250, max_velocity: float = 500, position_units: str = Units.MILLIMETERS):
+                 default_velocity: float = 250, max_velocity: float = 500, default_joint_velocity: float = 5.0, position_units: str = Units.MILLIMETERS):
         self.connection = KortexConnection(host, port, username=username, password=password, connect=connect)
         self.last_feedback = None
         self.last_feedback_time = 0
@@ -20,6 +20,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         self._joint_count: Optional[int] = None
         self._default_velocity = default_velocity
         self._max_velocity = max_velocity
+        self.default_joint_velocity = default_joint_velocity
 
     @property
     def position_units(self) -> str:
@@ -204,11 +205,14 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         self.connection.wait_for_action_end(timeout)
 
     def move_to_location(self, location: Location,
-                         velocity: Optional[float] = None, acceleration: Optional[float] = None,
+                         velocity: Optional[float] = None, acceleration: Optional[float] = None, frame: Optional[Frame] = None,
                          relative: bool = False, wait: bool = True, timeout: Optional[float] = None):
         if velocity is not None and velocity > self.max_velocity:
             raise robot_arms.RobotArmInvalidVelocityError(f'Invalid velocity: {velocity} m/s, must be less than {self.max_velocity} m/s')
 
+        if frame is not None:
+            raise robot_arms.RobotArmNotSupportedError(f'Frame parameter not supported')
+
         action = Base_pb2.Action()
 
         target_pose = action.reach_pose.target_pose
@@ -244,9 +248,6 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         if len(joint_positions) != self.joint_count:
             raise robot_arms.RobotArmInvalidJointsError(f'Invalid number of joint angles ({len(joint_positions)}), must be {self.joint_count}')
 
-        if velocity is not None and velocity > self.max_velocity:
-            raise robot_arms.RobotArmInvalidVelocityError(f'Invalid velocity: {velocity} m/s, must be less than {self.max_velocity} m/s')
-
         action = Base_pb2.Action()
 
         joint_offsets = self.joint_positions if relative else [0] * self.joint_count
@@ -257,16 +258,13 @@ class KinovaGen3Arm(robot_arms.RobotArm):
             joint_angle.value = joint_offsets[joint_id] + joint_positions[joint_id]
 
         action.reach_joint_angles.constraint.type = Base_pb2.JOINT_CONSTRAINT_SPEED
-        action.reach_joint_angles.constraint.value = self.default_velocity if velocity is None else velocity
+        action.reach_joint_angles.constraint.value = self.default_joint_velocity if velocity is None else velocity
 
         self.connection.execute_action(action, wait=wait, timeout=timeout)
 
     def move_joint(self, joint_id: int, position: float,
                       velocity: Optional[float] = None, acceleration: Optional[float] = None,
                       relative: bool = False, wait: bool = True, timeout: Optional[float] = None):
-        if velocity is not None and velocity > self.max_velocity:
-            raise robot_arms.RobotArmInvalidVelocityError(f'Invalid velocity: {velocity} m/s, must be less than {self.max_velocity} m/s')
-
         action = Base_pb2.Action()
 
         joint_offset = self.joint_positions[joint_id] if relative else 0
@@ -275,7 +273,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         joint_angle.value = joint_offset + position
 
         action.reach_joint_angles.constraint.type = Base_pb2.JOINT_CONSTRAINT_SPEED
-        action.reach_joint_angles.constraint.value = self.default_velocity if velocity is None else velocity
+        action.reach_joint_angles.constraint.value = self.default_joint_velocity if velocity is None else velocity
 
         self.connection.execute_action(action, wait=wait, timeout=timeout)
 
